import { Credentials } from 'n8n-core';
import type { ICredentialDataDecryptedObject, ICredentialsExpressionResolveValues, IHttpRequestOptions, INode, INodeCredentialsDetails, INodeProperties, IRequestOptionsSimplified, WorkflowExecuteMode, IHttpRequestHelper, IWorkflowExecuteAdditionalData, IExecuteData } from 'n8n-workflow';
import { ICredentialsHelper, Workflow } from 'n8n-workflow';
import type { CredentialsEntity } from './databases/entities/CredentialsEntity';
import { NodeTypes } from './NodeTypes';
import { CredentialTypes } from './CredentialTypes';
import { CredentialsOverwrites } from './CredentialsOverwrites';
import { Logger } from './Logger';
import { CredentialsRepository } from './databases/repositories/credentials.repository';
import { SharedCredentialsRepository } from './databases/repositories/sharedCredentials.repository';
export declare class CredentialsHelper extends ICredentialsHelper {
    private readonly logger;
    private readonly credentialTypes;
    private readonly nodeTypes;
    private readonly credentialsOverwrites;
    private readonly credentialsRepository;
    private readonly sharedCredentialsRepository;
    constructor(logger: Logger, credentialTypes: CredentialTypes, nodeTypes: NodeTypes, credentialsOverwrites: CredentialsOverwrites, credentialsRepository: CredentialsRepository, sharedCredentialsRepository: SharedCredentialsRepository);
    authenticate(credentials: ICredentialDataDecryptedObject, typeName: string, incomingRequestOptions: IHttpRequestOptions | IRequestOptionsSimplified, workflow: Workflow, node: INode): Promise<IHttpRequestOptions>;
    preAuthentication(helpers: IHttpRequestHelper, credentials: ICredentialDataDecryptedObject, typeName: string, node: INode, credentialsExpired: boolean): Promise<ICredentialDataDecryptedObject | undefined>;
    private resolveValue;
    getParentTypes(typeName: string): string[];
    getCredentials(nodeCredential: INodeCredentialsDetails, type: string, userId?: string): Promise<Credentials>;
    getCredentialsProperties(type: string): INodeProperties[];
    getDecrypted(additionalData: IWorkflowExecuteAdditionalData, nodeCredentials: INodeCredentialsDetails, type: string, mode: WorkflowExecuteMode, executeData?: IExecuteData, raw?: boolean, expressionResolveValues?: ICredentialsExpressionResolveValues): Promise<ICredentialDataDecryptedObject>;
    applyDefaultsAndOverwrites(additionalData: IWorkflowExecuteAdditionalData, decryptedDataOriginal: ICredentialDataDecryptedObject, type: string, mode: WorkflowExecuteMode, executeData?: IExecuteData, expressionResolveValues?: ICredentialsExpressionResolveValues, canUseSecrets?: boolean): ICredentialDataDecryptedObject;
    updateCredentials(nodeCredentials: INodeCredentialsDetails, type: string, data: ICredentialDataDecryptedObject): Promise<void>;
    credentialOwnedByOwner(nodeCredential: INodeCredentialsDetails): Promise<boolean>;
}
export declare function createCredentialsFromCredentialsEntity(credential: CredentialsEntity, encrypt?: boolean): Credentials;
