import type { TEntitlement, TFeatures, TLicenseBlock } from '@n8n_io/license-sdk';
import { InstanceSettings } from 'n8n-core';
import { Logger } from './Logger';
import { SettingsRepository } from './databases/repositories/settings.repository';
import type { BooleanLicenseFeature, N8nInstanceType, NumericLicenseFeature } from './Interfaces';
import { OrchestrationService } from './services/orchestration.service';
import { UsageMetricsService } from './services/usageMetrics.service';
type FeatureReturnType = Partial<{
    planName: string;
} & {
    [K in NumericLicenseFeature]: number;
} & {
    [K in BooleanLicenseFeature]: boolean;
}>;
export declare class License {
    private readonly logger;
    private readonly instanceSettings;
    private readonly orchestrationService;
    private readonly settingsRepository;
    private readonly usageMetricsService;
    private manager;
    private redisPublisher;
    private isShuttingDown;
    constructor(logger: Logger, instanceSettings: InstanceSettings, orchestrationService: OrchestrationService, settingsRepository: SettingsRepository, usageMetricsService: UsageMetricsService);
    init(instanceType?: N8nInstanceType): Promise<void>;
    loadCertStr(): Promise<TLicenseBlock>;
    onFeatureChange(_features: TFeatures): Promise<void>;
    saveCertStr(value: TLicenseBlock): Promise<void>;
    activate(activationKey: string): Promise<void>;
    reload(): Promise<void>;
    renew(): Promise<void>;
    shutdown(): Promise<void>;
    isFeatureEnabled(feature: BooleanLicenseFeature): boolean;
    isSharingEnabled(): boolean;
    isLogStreamingEnabled(): boolean;
    isLdapEnabled(): boolean;
    isSamlEnabled(): boolean;
    isAdvancedExecutionFiltersEnabled(): boolean;
    isAdvancedPermissionsLicensed(): boolean;
    isDebugInEditorLicensed(): boolean;
    isBinaryDataS3Licensed(): boolean;
    isMultipleMainInstancesLicensed(): boolean;
    isVariablesEnabled(): boolean;
    isSourceControlLicensed(): boolean;
    isExternalSecretsEnabled(): boolean;
    isWorkflowHistoryLicensed(): boolean;
    isAPIDisabled(): boolean;
    isWorkerViewLicensed(): boolean;
    getCurrentEntitlements(): TEntitlement[];
    getFeatureValue<T extends keyof FeatureReturnType>(feature: T): FeatureReturnType[T];
    getManagementJwt(): string;
    getMainPlan(): TEntitlement | undefined;
    getUsersLimit(): number;
    getTriggerLimit(): number;
    getVariablesLimit(): number;
    getWorkflowHistoryPruneLimit(): number;
    getPlanName(): string;
    getInfo(): string;
    isWithinUsersLimit(): boolean;
}
export {};
