"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Reset = void 0;
const typedi_1 = __importDefault(require("typedi"));
const constants_1 = require("../../Ldap/constants");
const authIdentity_repository_1 = require("../../databases/repositories/authIdentity.repository");
const authProviderSyncHistory_repository_1 = require("../../databases/repositories/authProviderSyncHistory.repository");
const settings_repository_1 = require("../../databases/repositories/settings.repository");
const user_repository_1 = require("../../databases/repositories/user.repository");
const BaseCommand_1 = require("../BaseCommand");
class Reset extends BaseCommand_1.BaseCommand {
    async run() {
        const ldapIdentities = await typedi_1.default.get(authIdentity_repository_1.AuthIdentityRepository).find({
            where: { providerType: 'ldap' },
            select: ['userId'],
        });
        await typedi_1.default.get(authProviderSyncHistory_repository_1.AuthProviderSyncHistoryRepository).delete({ providerType: 'ldap' });
        await typedi_1.default.get(authIdentity_repository_1.AuthIdentityRepository).delete({ providerType: 'ldap' });
        await typedi_1.default.get(user_repository_1.UserRepository).deleteMany(ldapIdentities.map((i) => i.userId));
        await typedi_1.default.get(settings_repository_1.SettingsRepository).delete({ key: constants_1.LDAP_FEATURE_NAME });
        await typedi_1.default.get(settings_repository_1.SettingsRepository).insert({
            key: constants_1.LDAP_FEATURE_NAME,
            value: JSON.stringify(constants_1.LDAP_DEFAULT_CONFIGURATION),
            loadOnStartup: true,
        });
        this.logger.info('Successfully reset the database to default ldap state.');
    }
    async catch(error) {
        this.logger.error('Error resetting database. See log messages for details.');
        this.logger.error(error.message);
    }
}
exports.Reset = Reset;
Reset.description = '\nResets the database to the default ldap state';
//# sourceMappingURL=reset.js.map