export declare const schema: {
    database: {
        type: {
            doc: string;
            format: readonly ["sqlite", "mariadb", "mysqldb", "postgresdb"];
            default: string;
            env: string;
        };
        tablePrefix: {
            doc: string;
            format: string;
            default: string;
            env: string;
        };
        logging: {
            enabled: {
                doc: string;
                format: BooleanConstructor;
                default: boolean;
                env: string;
            };
            options: {
                doc: string;
                format: StringConstructor;
                default: string;
                env: string;
            };
            maxQueryExecutionTime: {
                doc: string;
                format: NumberConstructor;
                default: number;
                env: string;
            };
        };
        postgresdb: {
            database: {
                doc: string;
                format: StringConstructor;
                default: string;
                env: string;
            };
            host: {
                doc: string;
                format: StringConstructor;
                default: string;
                env: string;
            };
            password: {
                doc: string;
                format: StringConstructor;
                default: string;
                env: string;
            };
            port: {
                doc: string;
                format: NumberConstructor;
                default: number;
                env: string;
            };
            user: {
                doc: string;
                format: StringConstructor;
                default: string;
                env: string;
            };
            schema: {
                doc: string;
                format: StringConstructor;
                default: string;
                env: string;
            };
            poolSize: {
                doc: string;
                format: NumberConstructor;
                default: number;
                env: string;
            };
            ssl: {
                enabled: {
                    doc: string;
                    format: BooleanConstructor;
                    default: boolean;
                    env: string;
                };
                ca: {
                    doc: string;
                    format: StringConstructor;
                    default: string;
                    env: string;
                };
                cert: {
                    doc: string;
                    format: StringConstructor;
                    default: string;
                    env: string;
                };
                key: {
                    doc: string;
                    format: StringConstructor;
                    default: string;
                    env: string;
                };
                rejectUnauthorized: {
                    doc: string;
                    format: BooleanConstructor;
                    default: boolean;
                    env: string;
                };
            };
        };
        mysqldb: {
            database: {
                doc: string;
                format: StringConstructor;
                default: string;
                env: string;
            };
            host: {
                doc: string;
                format: StringConstructor;
                default: string;
                env: string;
            };
            password: {
                doc: string;
                format: StringConstructor;
                default: string;
                env: string;
            };
            port: {
                doc: string;
                format: NumberConstructor;
                default: number;
                env: string;
            };
            user: {
                doc: string;
                format: StringConstructor;
                default: string;
                env: string;
            };
        };
        sqlite: {
            database: {
                doc: string;
                format: StringConstructor;
                default: string;
                env: string;
            };
            enableWAL: {
                doc: string;
                format: BooleanConstructor;
                default: boolean;
                env: string;
            };
            poolSize: {
                doc: string;
                format: NumberConstructor;
                default: number;
                env: string;
            };
            executeVacuumOnStartup: {
                doc: string;
                format: BooleanConstructor;
                default: boolean;
                env: string;
            };
        };
    };
    credentials: {
        overwrite: {
            data: {
                doc: string;
                format: string;
                default: string;
                env: string;
            };
            endpoint: {
                doc: string;
                format: StringConstructor;
                default: string;
                env: string;
            };
        };
        defaultName: {
            doc: string;
            format: StringConstructor;
            default: string;
            env: string;
        };
    };
    workflows: {
        defaultName: {
            doc: string;
            format: StringConstructor;
            default: string;
            env: string;
        };
        onboardingFlowDisabled: {
            doc: string;
            format: BooleanConstructor;
            default: boolean;
            env: string;
        };
        callerPolicyDefaultOption: {
            doc: string;
            format: readonly ["any", "none", "workflowsFromAList", "workflowsFromSameOwner"];
            default: string;
            env: string;
        };
    };
    executions: {
        process: {
            doc: string;
            format: StringConstructor;
            default: string;
            env: string;
        };
        mode: {
            doc: string;
            format: readonly ["regular", "queue"];
            default: string;
            env: string;
        };
        timeout: {
            doc: string;
            format: NumberConstructor;
            default: number;
            env: string;
        };
        maxTimeout: {
            doc: string;
            format: NumberConstructor;
            default: number;
            env: string;
        };
        saveDataOnError: {
            doc: string;
            format: readonly ["all", "none"];
            default: string;
            env: string;
        };
        saveDataOnSuccess: {
            doc: string;
            format: readonly ["all", "none"];
            default: string;
            env: string;
        };
        saveExecutionProgress: {
            doc: string;
            format: BooleanConstructor;
            default: boolean;
            env: string;
        };
        saveDataManualExecutions: {
            doc: string;
            format: BooleanConstructor;
            default: boolean;
            env: string;
        };
        pruneData: {
            doc: string;
            format: BooleanConstructor;
            default: boolean;
            env: string;
        };
        pruneDataMaxAge: {
            doc: string;
            format: NumberConstructor;
            default: number;
            env: string;
        };
        pruneDataHardDeleteBuffer: {
            doc: string;
            format: NumberConstructor;
            default: number;
            env: string;
        };
        pruneDataIntervals: {
            hardDelete: {
                doc: string;
                format: NumberConstructor;
                default: number;
                env: string;
            };
            softDelete: {
                doc: string;
                format: NumberConstructor;
                default: number;
                env: string;
            };
        };
        pruneDataMaxCount: {
            doc: string;
            format: NumberConstructor;
            default: number;
            env: string;
        };
    };
    queue: {
        health: {
            active: {
                doc: string;
                format: BooleanConstructor;
                default: boolean;
                env: string;
            };
            port: {
                doc: string;
                format: NumberConstructor;
                default: number;
                env: string;
            };
        };
        bull: {
            prefix: {
                doc: string;
                format: StringConstructor;
                default: string;
                env: string;
            };
            redis: {
                db: {
                    doc: string;
                    format: NumberConstructor;
                    default: number;
                    env: string;
                };
                host: {
                    doc: string;
                    format: StringConstructor;
                    default: string;
                    env: string;
                };
                password: {
                    doc: string;
                    format: StringConstructor;
                    default: string;
                    env: string;
                };
                port: {
                    doc: string;
                    format: NumberConstructor;
                    default: number;
                    env: string;
                };
                timeoutThreshold: {
                    doc: string;
                    format: NumberConstructor;
                    default: number;
                    env: string;
                };
                username: {
                    doc: string;
                    format: StringConstructor;
                    default: string;
                    env: string;
                };
                clusterNodes: {
                    doc: string;
                    format: StringConstructor;
                    default: string;
                    env: string;
                };
                tls: {
                    format: BooleanConstructor;
                    default: boolean;
                    env: string;
                    doc: string;
                };
            };
            queueRecoveryInterval: {
                doc: string;
                format: NumberConstructor;
                default: number;
                env: string;
            };
            gracefulShutdownTimeout: {
                doc: string;
                format: NumberConstructor;
                default: number;
                env: string;
            };
            settings: {
                lockDuration: {
                    doc: string;
                    format: NumberConstructor;
                    default: number;
                    env: string;
                };
                lockRenewTime: {
                    doc: string;
                    format: NumberConstructor;
                    default: number;
                    env: string;
                };
                stalledInterval: {
                    doc: string;
                    format: NumberConstructor;
                    default: number;
                    env: string;
                };
                maxStalledCount: {
                    doc: string;
                    format: NumberConstructor;
                    default: number;
                    env: string;
                };
            };
        };
    };
    generic: {
        timezone: {
            doc: string;
            format: string;
            default: string;
            env: string;
        };
        instanceType: {
            doc: string;
            format: readonly ["main", "webhook", "worker"];
            default: string;
        };
        releaseChannel: {
            doc: string;
            format: readonly ["stable", "beta", "nightly", "dev"];
            default: string;
            env: string;
        };
        gracefulShutdownTimeout: {
            doc: string;
            format: NumberConstructor;
            default: number;
            env: string;
        };
    };
    path: {
        format: StringConstructor;
        default: string;
        arg: string;
        env: string;
        doc: string;
    };
    host: {
        format: StringConstructor;
        default: string;
        arg: string;
        env: string;
        doc: string;
    };
    port: {
        format: NumberConstructor;
        default: number;
        arg: string;
        env: string;
        doc: string;
    };
    listen_address: {
        format: StringConstructor;
        default: string;
        env: string;
        doc: string;
    };
    protocol: {
        format: readonly ["http", "https"];
        default: string;
        env: string;
        doc: string;
    };
    secure_cookie: {
        doc: string;
        format: BooleanConstructor;
        default: boolean;
        env: string;
    };
    ssl_key: {
        format: StringConstructor;
        default: string;
        env: string;
        doc: string;
    };
    ssl_cert: {
        format: StringConstructor;
        default: string;
        env: string;
        doc: string;
    };
    editorBaseUrl: {
        format: StringConstructor;
        default: string;
        env: string;
        doc: string;
    };
    security: {
        restrictFileAccessTo: {
            doc: string;
            format: StringConstructor;
            default: string;
            env: string;
        };
        blockFileAccessToN8nFiles: {
            doc: string;
            format: BooleanConstructor;
            default: boolean;
            env: string;
        };
        audit: {
            daysAbandonedWorkflow: {
                doc: string;
                format: NumberConstructor;
                default: number;
                env: string;
            };
        };
    };
    endpoints: {
        payloadSizeMax: {
            format: NumberConstructor;
            default: number;
            env: string;
            doc: string;
        };
        metrics: {
            enable: {
                format: BooleanConstructor;
                default: boolean;
                env: string;
                doc: string;
            };
            prefix: {
                format: StringConstructor;
                default: string;
                env: string;
                doc: string;
            };
            includeDefaultMetrics: {
                format: BooleanConstructor;
                default: boolean;
                env: string;
                doc: string;
            };
            includeWorkflowIdLabel: {
                format: BooleanConstructor;
                default: boolean;
                env: string;
                doc: string;
            };
            includeNodeTypeLabel: {
                format: BooleanConstructor;
                default: boolean;
                env: string;
                doc: string;
            };
            includeCredentialTypeLabel: {
                format: BooleanConstructor;
                default: boolean;
                env: string;
                doc: string;
            };
            includeApiEndpoints: {
                format: BooleanConstructor;
                default: boolean;
                env: string;
                doc: string;
            };
            includeApiPathLabel: {
                format: BooleanConstructor;
                default: boolean;
                env: string;
                doc: string;
            };
            includeApiMethodLabel: {
                format: BooleanConstructor;
                default: boolean;
                env: string;
                doc: string;
            };
            includeApiStatusCodeLabel: {
                format: BooleanConstructor;
                default: boolean;
                env: string;
                doc: string;
            };
            includeCacheMetrics: {
                format: BooleanConstructor;
                default: boolean;
                env: string;
                doc: string;
            };
            includeMessageEventBusMetrics: {
                format: BooleanConstructor;
                default: boolean;
                env: string;
                doc: string;
            };
        };
        rest: {
            format: StringConstructor;
            default: string;
            env: string;
            doc: string;
        };
        form: {
            format: StringConstructor;
            default: string;
            env: string;
            doc: string;
        };
        formTest: {
            format: StringConstructor;
            default: string;
            env: string;
            doc: string;
        };
        formWaiting: {
            format: StringConstructor;
            default: string;
            env: string;
            doc: string;
        };
        webhook: {
            format: StringConstructor;
            default: string;
            env: string;
            doc: string;
        };
        webhookWaiting: {
            format: StringConstructor;
            default: string;
            env: string;
            doc: string;
        };
        webhookTest: {
            format: StringConstructor;
            default: string;
            env: string;
            doc: string;
        };
        disableUi: {
            format: BooleanConstructor;
            default: boolean;
            env: string;
            doc: string;
        };
        disableProductionWebhooksOnMainProcess: {
            format: BooleanConstructor;
            default: boolean;
            env: string;
            doc: string;
        };
        additionalNonUIRoutes: {
            doc: string;
            format: StringConstructor;
            default: string;
            env: string;
        };
    };
    publicApi: {
        disabled: {
            format: BooleanConstructor;
            default: boolean;
            env: string;
            doc: string;
        };
        path: {
            format: StringConstructor;
            default: string;
            env: string;
            doc: string;
        };
        swaggerUi: {
            disabled: {
                format: BooleanConstructor;
                default: boolean;
                env: string;
                doc: string;
            };
        };
    };
    workflowTagsDisabled: {
        format: BooleanConstructor;
        default: boolean;
        env: string;
        doc: string;
    };
    userManagement: {
        jwtSecret: {
            doc: string;
            format: StringConstructor;
            default: string;
            env: string;
        };
        jwtSessionDurationHours: {
            doc: string;
            format: NumberConstructor;
            default: number;
            env: string;
        };
        jwtRefreshTimeoutHours: {
            doc: string;
            format: NumberConstructor;
            default: number;
            env: string;
        };
        isInstanceOwnerSetUp: {
            doc: string;
            format: BooleanConstructor;
            default: boolean;
        };
        emails: {
            mode: {
                doc: string;
                format: readonly ["", "smtp"];
                default: string;
                env: string;
            };
            smtp: {
                host: {
                    doc: string;
                    format: StringConstructor;
                    default: string;
                    env: string;
                };
                port: {
                    doc: string;
                    format: NumberConstructor;
                    default: number;
                    env: string;
                };
                secure: {
                    doc: string;
                    format: BooleanConstructor;
                    default: boolean;
                    env: string;
                };
                auth: {
                    user: {
                        doc: string;
                        format: StringConstructor;
                        default: string;
                        env: string;
                    };
                    pass: {
                        doc: string;
                        format: StringConstructor;
                        default: string;
                        env: string;
                    };
                    serviceClient: {
                        doc: string;
                        format: StringConstructor;
                        default: string;
                        env: string;
                    };
                    privateKey: {
                        doc: string;
                        format: StringConstructor;
                        default: string;
                        env: string;
                    };
                };
                sender: {
                    doc: string;
                    format: StringConstructor;
                    default: string;
                    env: string;
                };
            };
            templates: {
                invite: {
                    doc: string;
                    format: StringConstructor;
                    default: string;
                    env: string;
                };
                passwordReset: {
                    doc: string;
                    format: StringConstructor;
                    default: string;
                    env: string;
                };
                workflowShared: {
                    doc: string;
                    format: StringConstructor;
                    default: string;
                    env: string;
                };
                credentialsShared: {
                    doc: string;
                    format: StringConstructor;
                    default: string;
                    env: string;
                };
            };
        };
        authenticationMethod: {
            doc: string;
            format: readonly ["email", "ldap", "saml"];
            default: string;
        };
    };
    externalFrontendHooksUrls: {
        doc: string;
        format: StringConstructor;
        default: string;
        env: string;
    };
    externalHookFiles: {
        doc: string;
        format: StringConstructor;
        default: string;
        env: string;
    };
    nodes: {
        include: {
            doc: string;
            format: string;
            default: undefined;
            env: string;
        };
        exclude: {
            doc: string;
            format: string;
            default: undefined;
            env: string;
        };
        errorTriggerType: {
            doc: string;
            format: StringConstructor;
            default: string;
            env: string;
        };
        communityPackages: {
            enabled: {
                doc: string;
                format: BooleanConstructor;
                default: boolean;
                env: string;
            };
        };
    };
    logs: {
        level: {
            doc: string;
            format: readonly ["silent", "error", "warn", "info", "debug", "verbose"];
            default: string;
            env: string;
        };
        output: {
            doc: string;
            format: StringConstructor;
            default: string;
            env: string;
        };
        file: {
            fileCountMax: {
                doc: string;
                format: NumberConstructor;
                default: number;
                env: string;
            };
            fileSizeMax: {
                doc: string;
                format: NumberConstructor;
                default: number;
                env: string;
            };
            location: {
                doc: string;
                format: StringConstructor;
                default: string;
                env: string;
            };
        };
    };
    versionNotifications: {
        enabled: {
            doc: string;
            format: BooleanConstructor;
            default: boolean;
            env: string;
        };
        endpoint: {
            doc: string;
            format: StringConstructor;
            default: string;
            env: string;
        };
        infoUrl: {
            doc: string;
            format: StringConstructor;
            default: string;
            env: string;
        };
    };
    templates: {
        enabled: {
            doc: string;
            format: BooleanConstructor;
            default: boolean;
            env: string;
        };
        host: {
            doc: string;
            format: StringConstructor;
            default: string;
            env: string;
        };
    };
    push: {
        backend: {
            format: readonly ["sse", "websocket"];
            default: string;
            env: string;
            doc: string;
        };
    };
    binaryDataManager: {
        availableModes: {
            format: string;
            default: string;
            env: string;
            doc: string;
        };
        mode: {
            format: readonly ["default", "filesystem", "s3"];
            default: string;
            env: string;
            doc: string;
        };
        localStoragePath: {
            format: StringConstructor;
            default: string;
            env: string;
            doc: string;
        };
    };
    externalStorage: {
        s3: {
            host: {
                format: StringConstructor;
                default: string;
                env: string;
                doc: string;
            };
            bucket: {
                name: {
                    format: StringConstructor;
                    default: string;
                    env: string;
                    doc: string;
                };
                region: {
                    format: StringConstructor;
                    default: string;
                    env: string;
                    doc: string;
                };
            };
            credentials: {
                accessKey: {
                    format: StringConstructor;
                    default: string;
                    env: string;
                    doc: string;
                };
                accessSecret: {
                    format: StringConstructor;
                    default: string;
                    env: string;
                    doc: string;
                };
            };
        };
    };
    externalSecrets: {
        updateInterval: {
            format: NumberConstructor;
            default: number;
            env: string;
            doc: string;
        };
        preferGet: {
            format: BooleanConstructor;
            default: boolean;
            env: string;
            doc: string;
        };
    };
    deployment: {
        type: {
            format: StringConstructor;
            default: string;
            env: string;
        };
    };
    mfa: {
        enabled: {
            format: BooleanConstructor;
            default: boolean;
            doc: string;
            env: string;
        };
    };
    sso: {
        justInTimeProvisioning: {
            format: BooleanConstructor;
            default: boolean;
            doc: string;
        };
        redirectLoginToSso: {
            format: BooleanConstructor;
            default: boolean;
            doc: string;
        };
        saml: {
            loginEnabled: {
                format: BooleanConstructor;
                default: boolean;
                doc: string;
            };
            loginLabel: {
                format: StringConstructor;
                default: string;
            };
        };
        ldap: {
            loginEnabled: {
                format: BooleanConstructor;
                default: boolean;
            };
            loginLabel: {
                format: StringConstructor;
                default: string;
            };
        };
    };
    hiringBanner: {
        enabled: {
            doc: string;
            format: BooleanConstructor;
            default: boolean;
            env: string;
        };
    };
    personalization: {
        enabled: {
            doc: string;
            format: BooleanConstructor;
            default: boolean;
            env: string;
        };
    };
    diagnostics: {
        enabled: {
            doc: string;
            format: BooleanConstructor;
            default: boolean;
            env: string;
        };
        config: {
            posthog: {
                apiKey: {
                    doc: string;
                    format: StringConstructor;
                    default: string;
                    env: string;
                };
                apiHost: {
                    doc: string;
                    format: StringConstructor;
                    default: string;
                    env: string;
                };
            };
            sentry: {
                dsn: {
                    doc: string;
                    format: StringConstructor;
                    default: string;
                    env: string;
                };
            };
            frontend: {
                doc: string;
                format: StringConstructor;
                default: string;
                env: string;
            };
            backend: {
                doc: string;
                format: StringConstructor;
                default: string;
                env: string;
            };
        };
    };
    defaultLocale: {
        doc: string;
        format: StringConstructor;
        default: string;
        env: string;
    };
    onboardingCallPrompt: {
        enabled: {
            doc: string;
            format: BooleanConstructor;
            default: boolean;
            env: string;
        };
    };
    license: {
        serverUrl: {
            format: StringConstructor;
            default: string;
            env: string;
            doc: string;
        };
        autoRenewEnabled: {
            format: BooleanConstructor;
            default: boolean;
            env: string;
            doc: string;
        };
        autoRenewOffset: {
            format: NumberConstructor;
            default: number;
            env: string;
            doc: string;
        };
        activationKey: {
            format: StringConstructor;
            default: string;
            env: string;
            doc: string;
        };
        tenantId: {
            format: NumberConstructor;
            default: number;
            env: string;
            doc: string;
        };
        cert: {
            format: StringConstructor;
            default: string;
            env: string;
            doc: string;
        };
    };
    hideUsagePage: {
        format: BooleanConstructor;
        default: boolean;
        env: string;
        doc: string;
    };
    eventBus: {
        checkUnsentInterval: {
            doc: string;
            format: NumberConstructor;
            default: number;
            env: string;
        };
        logWriter: {
            keepLogCount: {
                doc: string;
                format: NumberConstructor;
                default: number;
                env: string;
            };
            maxFileSizeInKB: {
                doc: string;
                format: NumberConstructor;
                default: number;
                env: string;
            };
            logBaseName: {
                doc: string;
                format: StringConstructor;
                default: string;
                env: string;
            };
        };
        crashRecoveryMode: {
            doc: string;
            format: readonly ["simple", "extensive"];
            default: string;
            env: string;
        };
    };
    redis: {
        prefix: {
            doc: string;
            format: StringConstructor;
            default: string;
            env: string;
        };
        queueModeId: {
            doc: string;
            format: StringConstructor;
            default: string;
        };
    };
    cache: {
        backend: {
            doc: string;
            format: readonly ["memory", "redis", "auto"];
            default: string;
            env: string;
        };
        memory: {
            maxSize: {
                doc: string;
                format: NumberConstructor;
                default: number;
                env: string;
            };
            ttl: {
                doc: string;
                format: NumberConstructor;
                default: number;
                env: string;
            };
        };
        redis: {
            prefix: {
                doc: string;
                format: StringConstructor;
                default: string;
                env: string;
            };
            ttl: {
                doc: string;
                format: NumberConstructor;
                default: number;
                env: string;
            };
        };
    };
    ai: {
        enabled: {
            doc: string;
            format: BooleanConstructor;
            default: boolean;
            env: string;
        };
        provider: {
            doc: string;
            format: StringConstructor;
            default: string;
            env: string;
        };
        openAIApiKey: {
            doc: string;
            format: StringConstructor;
            default: string;
            env: string;
        };
    };
    expression: {
        evaluator: {
            doc: string;
            format: readonly ["tmpl", "tournament"];
            default: string;
            env: string;
        };
        reportDifference: {
            doc: string;
            format: BooleanConstructor;
            default: boolean;
            env: string;
        };
    };
    sourceControl: {
        defaultKeyPairType: {
            doc: string;
            format: readonly ["rsa", "ed25519"];
            default: string;
            env: string;
        };
    };
    workflowHistory: {
        enabled: {
            doc: string;
            format: BooleanConstructor;
            default: boolean;
            env: string;
        };
        pruneTime: {
            doc: string;
            format: NumberConstructor;
            default: number;
            env: string;
        };
    };
    multiMainSetup: {
        instanceType: {
            doc: string;
            format: readonly ["unset", "leader", "follower"];
            default: string;
        };
        enabled: {
            doc: string;
            format: BooleanConstructor;
            default: boolean;
            env: string;
        };
        ttl: {
            doc: string;
            format: NumberConstructor;
            default: number;
            env: string;
        };
        interval: {
            doc: string;
            format: NumberConstructor;
            default: number;
            env: string;
        };
    };
    proxy_hops: {
        format: NumberConstructor;
        default: number;
        env: string;
        doc: string;
    };
};
