import { AuthService } from '../auth/auth.service';
import { Request, Response } from 'express';
import { AuthenticatedRequest, LoginRequest, UserRequest } from '../requests';
import type { PublicUser } from '../Interfaces';
import { PostHogClient } from '../posthog';
import { InternalHooks } from '../InternalHooks';
import { License } from '../License';
import { UserService } from '../services/user.service';
import { MfaService } from '../Mfa/mfa.service';
import { Logger } from '../Logger';
import { UserRepository } from '../databases/repositories/user.repository';
export declare class AuthController {
    private readonly logger;
    private readonly internalHooks;
    private readonly authService;
    private readonly mfaService;
    private readonly userService;
    private readonly license;
    private readonly userRepository;
    private readonly postHog?;
    constructor(logger: Logger, internalHooks: InternalHooks, authService: AuthService, mfaService: MfaService, userService: UserService, license: License, userRepository: UserRepository, postHog?: PostHogClient | undefined);
    login(req: LoginRequest, res: Response): Promise<PublicUser | undefined>;
    currentUser(req: AuthenticatedRequest): Promise<PublicUser>;
    resolveSignupToken(req: UserRequest.ResolveSignUp): Promise<{
        inviter: {
            firstName: string;
            lastName: string;
        };
    }>;
    logout(_: Request, res: Response): {
        loggedOut: boolean;
    };
    private validateMfaToken;
    private validateMfaRecoveryCode;
}
