"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DynamicNodeParametersController = void 0;
const n8n_workflow_1 = require("n8n-workflow");
const decorators_1 = require("../decorators");
const WorkflowExecuteAdditionalData_1 = require("../WorkflowExecuteAdditionalData");
const dynamicNodeParameters_service_1 = require("../services/dynamicNodeParameters.service");
const bad_request_error_1 = require("../errors/response-errors/bad-request.error");
const assertMethodName = (req, res, next) => {
    const { methodName } = req.query;
    if (!methodName) {
        throw new bad_request_error_1.BadRequestError('Parameter methodName is required.');
    }
    next();
};
let DynamicNodeParametersController = class DynamicNodeParametersController {
    constructor(service) {
        this.service = service;
    }
    parseQueryParams(req, _, next) {
        const { credentials, currentNodeParameters, nodeTypeAndVersion } = req.query;
        if (!nodeTypeAndVersion) {
            throw new bad_request_error_1.BadRequestError('Parameter nodeTypeAndVersion is required.');
        }
        if (!currentNodeParameters) {
            throw new bad_request_error_1.BadRequestError('Parameter currentNodeParameters is required.');
        }
        req.params = {
            nodeTypeAndVersion: (0, n8n_workflow_1.jsonParse)(nodeTypeAndVersion),
            currentNodeParameters: (0, n8n_workflow_1.jsonParse)(currentNodeParameters),
            credentials: credentials ? (0, n8n_workflow_1.jsonParse)(credentials) : undefined,
        };
        next();
    }
    async getOptions(req) {
        const { path, methodName, loadOptions } = req.query;
        const { credentials, currentNodeParameters, nodeTypeAndVersion } = req.params;
        const additionalData = await (0, WorkflowExecuteAdditionalData_1.getBase)(req.user.id, currentNodeParameters);
        if (methodName) {
            return await this.service.getOptionsViaMethodName(methodName, path, additionalData, nodeTypeAndVersion, currentNodeParameters, credentials);
        }
        if (loadOptions) {
            return await this.service.getOptionsViaLoadOptions((0, n8n_workflow_1.jsonParse)(loadOptions), additionalData, nodeTypeAndVersion, currentNodeParameters, credentials);
        }
        return [];
    }
    async getResourceLocatorResults(req) {
        const { path, methodName, filter, paginationToken } = req.query;
        const { credentials, currentNodeParameters, nodeTypeAndVersion } = req.params;
        const additionalData = await (0, WorkflowExecuteAdditionalData_1.getBase)(req.user.id, currentNodeParameters);
        return await this.service.getResourceLocatorResults(methodName, path, additionalData, nodeTypeAndVersion, currentNodeParameters, credentials, filter, paginationToken);
    }
    async getResourceMappingFields(req) {
        const { path, methodName } = req.query;
        const { credentials, currentNodeParameters, nodeTypeAndVersion } = req.params;
        const additionalData = await (0, WorkflowExecuteAdditionalData_1.getBase)(req.user.id, currentNodeParameters);
        return await this.service.getResourceMappingFields(methodName, path, additionalData, nodeTypeAndVersion, currentNodeParameters, credentials);
    }
};
exports.DynamicNodeParametersController = DynamicNodeParametersController;
__decorate([
    (0, decorators_1.Middleware)(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Function]),
    __metadata("design:returntype", void 0)
], DynamicNodeParametersController.prototype, "parseQueryParams", null);
__decorate([
    (0, decorators_1.Get)('/options'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], DynamicNodeParametersController.prototype, "getOptions", null);
__decorate([
    (0, decorators_1.Get)('/resource-locator-results', { middlewares: [assertMethodName] }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], DynamicNodeParametersController.prototype, "getResourceLocatorResults", null);
__decorate([
    (0, decorators_1.Get)('/resource-mapper-fields', { middlewares: [assertMethodName] }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], DynamicNodeParametersController.prototype, "getResourceMappingFields", null);
exports.DynamicNodeParametersController = DynamicNodeParametersController = __decorate([
    (0, decorators_1.RestController)('/dynamic-node-parameters'),
    __metadata("design:paramtypes", [dynamicNodeParameters_service_1.DynamicNodeParametersService])
], DynamicNodeParametersController);
//# sourceMappingURL=dynamicNodeParameters.controller.js.map