import { AuthService } from '../auth/auth.service';
import { ListQuery, UserRequest } from '../requests';
import { ActiveWorkflowRunner } from '../ActiveWorkflowRunner';
import type { PublicUser } from '../Interfaces';
import { SharedCredentialsRepository } from '../databases/repositories/sharedCredentials.repository';
import { SharedWorkflowRepository } from '../databases/repositories/sharedWorkflow.repository';
import { UserRepository } from '../databases/repositories/user.repository';
import { UserService } from '../services/user.service';
import { Logger } from '../Logger';
import { ExternalHooks } from '../ExternalHooks';
import { InternalHooks } from '../InternalHooks';
export declare class UsersController {
    private readonly logger;
    private readonly externalHooks;
    private readonly internalHooks;
    private readonly sharedCredentialsRepository;
    private readonly sharedWorkflowRepository;
    private readonly userRepository;
    private readonly activeWorkflowRunner;
    private readonly authService;
    private readonly userService;
    constructor(logger: Logger, externalHooks: ExternalHooks, internalHooks: InternalHooks, sharedCredentialsRepository: SharedCredentialsRepository, sharedWorkflowRepository: SharedWorkflowRepository, userRepository: UserRepository, activeWorkflowRunner: ActiveWorkflowRunner, authService: AuthService, userService: UserService);
    static ERROR_MESSAGES: {
        readonly CHANGE_ROLE: {
            readonly NO_USER: "Target user not found";
            readonly NO_ADMIN_ON_OWNER: "Admin cannot change role on global owner";
            readonly NO_OWNER_ON_OWNER: "Owner cannot change role on global owner";
        };
    };
    private removeSupplementaryFields;
    listUsers(req: ListQuery.Request): Promise<Partial<PublicUser>[]>;
    getUserPasswordResetLink(req: UserRequest.PasswordResetLink): Promise<{
        link: string;
    }>;
    updateUserSettings(req: UserRequest.UserSettingsUpdate): Promise<import("n8n-workflow").IUserSettings | null>;
    deleteUser(req: UserRequest.Delete): Promise<{
        success: boolean;
    }>;
    changeGlobalRole(req: UserRequest.ChangeRole): Promise<{
        success: boolean;
    }>;
}
