import { CredentialsService } from './credentials.service';
import { CredentialRequest, ListQuery } from '../requests';
import { InternalHooks } from '../InternalHooks';
import { Logger } from '../Logger';
import { NamingService } from '../services/naming.service';
import { License } from '../License';
import { CredentialsRepository } from '../databases/repositories/credentials.repository';
import { OwnershipService } from '../services/ownership.service';
import { EnterpriseCredentialsService } from './credentials.service.ee';
import { UserManagementMailer } from '../UserManagement/email';
export declare class CredentialsController {
    private readonly credentialsService;
    private readonly enterpriseCredentialsService;
    private readonly credentialsRepository;
    private readonly namingService;
    private readonly license;
    private readonly logger;
    private readonly ownershipService;
    private readonly internalHooks;
    private readonly userManagementMailer;
    constructor(credentialsService: CredentialsService, enterpriseCredentialsService: EnterpriseCredentialsService, credentialsRepository: CredentialsRepository, namingService: NamingService, license: License, logger: Logger, ownershipService: OwnershipService, internalHooks: InternalHooks, userManagementMailer: UserManagementMailer);
    getMany(req: ListQuery.Request): Promise<import("../databases/entities/CredentialsEntity").CredentialsEntity[]>;
    generateUniqueName(req: CredentialRequest.NewName): Promise<{
        name: string;
    }>;
    getOne(req: CredentialRequest.Get): Promise<{
        name: string;
        type: string;
        shared: import("../databases/entities/SharedCredentials").SharedCredentials[];
        id: string;
        createdAt: Date;
        updatedAt: Date;
    } | {
        name: string;
        type: string;
        shared: import("../databases/entities/SharedCredentials").SharedCredentials[];
        id: string;
        createdAt: Date;
        updatedAt: Date;
        data: import("n8n-workflow").ICredentialDataDecryptedObject;
    }>;
    testCredentials(req: CredentialRequest.Test): Promise<import("n8n-workflow").INodeCredentialTestResult>;
    createCredentials(req: CredentialRequest.Create): Promise<import("../databases/entities/CredentialsEntity").CredentialsEntity>;
    updateCredentials(req: CredentialRequest.Update): Promise<{
        name: string;
        type: string;
        shared: import("../databases/entities/SharedCredentials").SharedCredentials[];
        id: string;
        createdAt: Date;
        updatedAt: Date;
    }>;
    deleteCredentials(req: CredentialRequest.Delete): Promise<boolean>;
    shareCredentials(req: CredentialRequest.Share): Promise<void>;
}
