import type { ICredentialDataDecryptedObject, ICredentialsDecrypted } from 'n8n-workflow';
import type { FindOptionsWhere } from '@n8n/typeorm';
import type { Scope } from '@n8n/permissions';
import type { ICredentialsDb } from '../Interfaces';
import { CredentialsEntity } from '../databases/entities/CredentialsEntity';
import { SharedCredentials } from '../databases/entities/SharedCredentials';
import { ExternalHooks } from '../ExternalHooks';
import type { User } from '../databases/entities/User';
import type { CredentialRequest, ListQuery } from '../requests';
import { CredentialTypes } from '../CredentialTypes';
import { OwnershipService } from '../services/ownership.service';
import { Logger } from '../Logger';
import { CredentialsRepository } from '../databases/repositories/credentials.repository';
import { SharedCredentialsRepository } from '../databases/repositories/sharedCredentials.repository';
import { CredentialsTester } from '../services/credentials-tester.service';
export type CredentialsGetSharedOptions = {
    allowGlobalScope: true;
    globalScope: Scope;
} | {
    allowGlobalScope: false;
};
export declare class CredentialsService {
    private readonly credentialsRepository;
    private readonly sharedCredentialsRepository;
    private readonly ownershipService;
    private readonly logger;
    private readonly credentialsTester;
    private readonly externalHooks;
    private readonly credentialTypes;
    constructor(credentialsRepository: CredentialsRepository, sharedCredentialsRepository: SharedCredentialsRepository, ownershipService: OwnershipService, logger: Logger, credentialsTester: CredentialsTester, externalHooks: ExternalHooks, credentialTypes: CredentialTypes);
    get(where: FindOptionsWhere<ICredentialsDb>, options?: {
        relations: string[];
    }): Promise<CredentialsEntity | null>;
    getMany(user: User, options?: {
        listQueryOptions?: ListQuery.Options;
        onlyOwn?: boolean;
    }): Promise<CredentialsEntity[]>;
    getSharing(user: User, credentialId: string, options: CredentialsGetSharedOptions, relations?: string[]): Promise<SharedCredentials | null>;
    prepareCreateData(data: CredentialRequest.CredentialProperties): Promise<CredentialsEntity>;
    prepareUpdateData(data: CredentialRequest.CredentialProperties, decryptedData: ICredentialDataDecryptedObject): Promise<CredentialsEntity>;
    createEncryptedData(credentialId: string | null, data: CredentialsEntity): ICredentialsDb;
    decrypt(credential: CredentialsEntity): ICredentialDataDecryptedObject;
    update(credentialId: string, newCredentialData: ICredentialsDb): Promise<CredentialsEntity | null>;
    save(credential: CredentialsEntity, encryptedData: ICredentialsDb, user: User): Promise<CredentialsEntity>;
    delete(credentials: CredentialsEntity): Promise<void>;
    test(user: User, credentials: ICredentialsDecrypted): Promise<import("n8n-workflow").INodeCredentialTestResult>;
    redact(data: ICredentialDataDecryptedObject, credential: CredentialsEntity): ICredentialDataDecryptedObject;
    private unredactRestoreValues;
    unredact(redactedData: ICredentialDataDecryptedObject, savedData: ICredentialDataDecryptedObject): ICredentialDataDecryptedObject;
}
