"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EnterpriseCredentialsService = void 0;
const sharedCredentials_repository_1 = require("../databases/repositories/sharedCredentials.repository");
const user_repository_1 = require("../databases/repositories/user.repository");
const CredentialsEntity_1 = require("../databases/entities/CredentialsEntity");
const typedi_1 = require("typedi");
let EnterpriseCredentialsService = class EnterpriseCredentialsService {
    constructor(userRepository, sharedCredentialsRepository) {
        this.userRepository = userRepository;
        this.sharedCredentialsRepository = sharedCredentialsRepository;
    }
    async isOwned(user, credentialId) {
        const sharing = await this.getSharing(user, credentialId, { allowGlobalScope: false }, [
            'credentials',
        ]);
        if (!sharing || sharing.role !== 'credential:owner')
            return { ownsCredential: false };
        const { credentials: credential } = sharing;
        return { ownsCredential: true, credential };
    }
    async getSharing(user, credentialId, options, relations = ['credentials']) {
        const where = { credentialsId: credentialId };
        if (!options.allowGlobalScope || !user.hasGlobalScope(options.globalScope)) {
            where.userId = user.id;
        }
        return await this.sharedCredentialsRepository.findOne({
            where,
            relations,
        });
    }
    async getSharings(transaction, credentialId, relations = ['shared']) {
        var _a;
        const credential = await transaction.findOne(CredentialsEntity_1.CredentialsEntity, {
            where: { id: credentialId },
            relations,
        });
        return (_a = credential === null || credential === void 0 ? void 0 : credential.shared) !== null && _a !== void 0 ? _a : [];
    }
    async share(transaction, credential, shareWithIds) {
        const users = await this.userRepository.getByIds(transaction, shareWithIds);
        const newSharedCredentials = users
            .filter((user) => !user.isPending)
            .map((user) => this.sharedCredentialsRepository.create({
            credentialsId: credential.id,
            userId: user.id,
            role: 'credential:user',
        }));
        return await transaction.save(newSharedCredentials);
    }
};
exports.EnterpriseCredentialsService = EnterpriseCredentialsService;
exports.EnterpriseCredentialsService = EnterpriseCredentialsService = __decorate([
    (0, typedi_1.Service)(),
    __metadata("design:paramtypes", [user_repository_1.UserRepository,
        sharedCredentials_repository_1.SharedCredentialsRepository])
], EnterpriseCredentialsService);
//# sourceMappingURL=credentials.service.ee.js.map