import { DataSource, Repository, type UpdateResult, type FindOptionsWhere, type EntityManager, type DeleteResult } from '@n8n/typeorm';
import type { ListQuery } from '../../requests';
import { WorkflowEntity } from '../entities/WorkflowEntity';
import { SharedWorkflow } from '../entities/SharedWorkflow';
export declare class WorkflowRepository extends Repository<WorkflowEntity> {
    constructor(dataSource: DataSource);
    get(where: FindOptionsWhere<WorkflowEntity>, options?: {
        relations: string[];
    }): Promise<WorkflowEntity | null>;
    getAllActive(): Promise<WorkflowEntity[]>;
    getActiveIds(): Promise<string[]>;
    findById(workflowId: string): Promise<WorkflowEntity | null>;
    findByIds(workflowIds: string[], { fields }?: {
        fields?: string[];
    }): Promise<WorkflowEntity[]>;
    getActiveTriggerCount(): Promise<number>;
    getSharings(transaction: EntityManager, workflowId: string, relations?: string[]): Promise<SharedWorkflow[]>;
    pruneSharings(transaction: EntityManager, workflowId: string, userIds: string[]): Promise<DeleteResult>;
    updateWorkflowTriggerCount(id: string, triggerCount: number): Promise<UpdateResult>;
    getMany(sharedWorkflowIds: string[], options?: ListQuery.Options): Promise<{
        workflows: (Pick<WorkflowEntity, "id"> & Partial<Pick<WorkflowEntity, "name" | "tags" | "active" | "createdAt" | "updatedAt" | "versionId">>)[] | ListQuery.Workflow.WithSharing[];
        count: number;
    }>;
    findStartingWith(workflowName: string): Promise<Array<{
        name: string;
    }>>;
    findIn(workflowIds: string[]): Promise<WorkflowEntity[]>;
    findWebhookBasedActiveWorkflows(): Promise<{
        id: string;
        name: string;
    }[]>;
    updateActiveState(workflowId: string, newState: boolean): Promise<UpdateResult>;
    deactivateAll(): Promise<UpdateResult>;
    activateAll(): Promise<UpdateResult>;
    findByActiveState(activeState: boolean): Promise<WorkflowEntity[]>;
}
