import type { ExecutionError, INode, WorkflowExecuteMode } from 'n8n-workflow';
import { Workflow } from 'n8n-workflow';
import { ActiveExecutions } from '../ActiveExecutions';
import type { IExecutionFlattedResponse, IExecutionResponse, IWorkflowDb } from '../Interfaces';
import { NodeTypes } from '../NodeTypes';
import { Queue } from '../Queue';
import type { ExecutionRequest } from './execution.types';
import { WorkflowRunner } from '../WorkflowRunner';
import { ExecutionRepository } from '../databases/repositories/execution.repository';
import { WorkflowRepository } from '../databases/repositories/workflow.repository';
import { Logger } from '../Logger';
export declare class ExecutionService {
    private readonly logger;
    private readonly queue;
    private readonly activeExecutions;
    private readonly executionRepository;
    private readonly workflowRepository;
    private readonly nodeTypes;
    private readonly workflowRunner;
    constructor(logger: Logger, queue: Queue, activeExecutions: ActiveExecutions, executionRepository: ExecutionRepository, workflowRepository: WorkflowRepository, nodeTypes: NodeTypes, workflowRunner: WorkflowRunner);
    findMany(req: ExecutionRequest.GetMany, sharedWorkflowIds: string[]): Promise<{
        count: number;
        results: import("n8n-workflow").ExecutionSummary[];
        estimated: boolean;
    }>;
    findOne(req: ExecutionRequest.GetOne, sharedWorkflowIds: string[]): Promise<IExecutionResponse | IExecutionFlattedResponse | undefined>;
    retry(req: ExecutionRequest.Retry, sharedWorkflowIds: string[]): Promise<boolean>;
    delete(req: ExecutionRequest.Delete, sharedWorkflowIds: string[]): Promise<void>;
    createErrorExecution(error: ExecutionError, node: INode, workflowData: IWorkflowDb, workflow: Workflow, mode: WorkflowExecuteMode): Promise<void>;
}
