"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExecutionsController = void 0;
const execution_service_1 = require("./execution.service");
const decorators_1 = require("../decorators");
const execution_service_ee_1 = require("./execution.service.ee");
const License_1 = require("../License");
const workflowSharing_service_1 = require("../workflows/workflowSharing.service");
const config_1 = __importDefault(require("../config"));
const n8n_workflow_1 = require("n8n-workflow");
const not_found_error_1 = require("../errors/response-errors/not-found.error");
const active_execution_service_1 = require("./active-execution.service");
let ExecutionsController = class ExecutionsController {
    constructor(executionService, enterpriseExecutionService, workflowSharingService, activeExecutionService, license) {
        this.executionService = executionService;
        this.enterpriseExecutionService = enterpriseExecutionService;
        this.workflowSharingService = workflowSharingService;
        this.activeExecutionService = activeExecutionService;
        this.license = license;
        this.isQueueMode = config_1.default.getEnv('executions.mode') === 'queue';
    }
    async getAccessibleWorkflowIds(user) {
        return this.license.isSharingEnabled()
            ? await this.workflowSharingService.getSharedWorkflowIds(user)
            : await this.workflowSharingService.getSharedWorkflowIds(user, ['workflow:owner']);
    }
    async getMany(req) {
        const workflowIds = await this.getAccessibleWorkflowIds(req.user);
        if (workflowIds.length === 0)
            return { count: 0, estimated: false, results: [] };
        return await this.executionService.findMany(req, workflowIds);
    }
    async getActive(req) {
        var _a;
        const filter = ((_a = req.query.filter) === null || _a === void 0 ? void 0 : _a.length) ? (0, n8n_workflow_1.jsonParse)(req.query.filter) : {};
        const workflowIds = await this.getAccessibleWorkflowIds(req.user);
        return this.isQueueMode
            ? await this.activeExecutionService.findManyInQueueMode(filter, workflowIds)
            : await this.activeExecutionService.findManyInRegularMode(filter, workflowIds);
    }
    async stop(req) {
        const workflowIds = await this.getAccessibleWorkflowIds(req.user);
        if (workflowIds.length === 0)
            throw new not_found_error_1.NotFoundError('Execution not found');
        const execution = await this.activeExecutionService.findOne(req.params.id, workflowIds);
        if (!execution)
            throw new not_found_error_1.NotFoundError('Execution not found');
        return await this.activeExecutionService.stop(execution);
    }
    async getOne(req) {
        const workflowIds = await this.getAccessibleWorkflowIds(req.user);
        if (workflowIds.length === 0)
            throw new not_found_error_1.NotFoundError('Execution not found');
        return this.license.isSharingEnabled()
            ? await this.enterpriseExecutionService.findOne(req, workflowIds)
            : await this.executionService.findOne(req, workflowIds);
    }
    async retry(req) {
        const workflowIds = await this.getAccessibleWorkflowIds(req.user);
        if (workflowIds.length === 0)
            throw new not_found_error_1.NotFoundError('Execution not found');
        return await this.executionService.retry(req, workflowIds);
    }
    async delete(req) {
        const workflowIds = await this.getAccessibleWorkflowIds(req.user);
        if (workflowIds.length === 0)
            throw new not_found_error_1.NotFoundError('Execution not found');
        return await this.executionService.delete(req, workflowIds);
    }
};
exports.ExecutionsController = ExecutionsController;
__decorate([
    (0, decorators_1.Get)('/'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], ExecutionsController.prototype, "getMany", null);
__decorate([
    (0, decorators_1.Get)('/active'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], ExecutionsController.prototype, "getActive", null);
__decorate([
    (0, decorators_1.Post)('/active/:id/stop'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], ExecutionsController.prototype, "stop", null);
__decorate([
    (0, decorators_1.Get)('/:id'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], ExecutionsController.prototype, "getOne", null);
__decorate([
    (0, decorators_1.Post)('/:id/retry'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], ExecutionsController.prototype, "retry", null);
__decorate([
    (0, decorators_1.Post)('/delete'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], ExecutionsController.prototype, "delete", null);
exports.ExecutionsController = ExecutionsController = __decorate([
    (0, decorators_1.RestController)('/executions'),
    __metadata("design:paramtypes", [execution_service_1.ExecutionService,
        execution_service_ee_1.EnterpriseExecutionsService,
        workflowSharing_service_1.WorkflowSharingService,
        active_execution_service_1.ActiveExecutionService,
        License_1.License])
], ExecutionsController);
//# sourceMappingURL=executions.controller.js.map