import type { IPushDataType, IPushDataWorkerStatusPayload, IWorkflowDb } from '../../Interfaces';
export type RedisServiceCommand = 'getStatus' | 'getId' | 'restartEventBus' | 'stopWorker' | 'reloadLicense' | 'reloadExternalSecretsProviders' | 'display-workflow-activation' | 'display-workflow-deactivation' | 'add-webhooks-triggers-and-pollers' | 'remove-triggers-and-pollers' | 'workflow-failed-to-activate' | 'relay-execution-lifecycle-event' | 'clear-test-webhooks';
export type RedisServiceBaseCommand = {
    senderId: string;
    command: Exclude<RedisServiceCommand, 'relay-execution-lifecycle-event' | 'clear-test-webhooks'>;
    payload?: {
        [key: string]: string | number | boolean | string[] | number[] | boolean[];
    };
} | {
    senderId: string;
    command: 'relay-execution-lifecycle-event';
    payload: {
        type: IPushDataType;
        args: Record<string, unknown>;
        pushRef: string;
    };
} | {
    senderId: string;
    command: 'clear-test-webhooks';
    payload: {
        webhookKey: string;
        workflowEntity: IWorkflowDb;
        pushRef: string;
    };
};
export type RedisServiceWorkerResponseObject = {
    workerId: string;
} & (RedisServiceBaseCommand | {
    command: 'getStatus';
    payload: IPushDataWorkerStatusPayload;
} | {
    command: 'getId';
} | {
    command: 'restartEventBus';
    payload: {
        result: 'success' | 'error';
        error?: string;
    };
} | {
    command: 'reloadExternalSecretsProviders';
    payload: {
        result: 'success' | 'error';
        error?: string;
    };
} | {
    command: 'stopWorker';
} | {
    command: 'workflowActiveStateChanged';
    payload: {
        oldState: boolean;
        newState: boolean;
        workflowId: string;
    };
});
export type RedisServiceCommandObject = {
    targets?: string[];
} & RedisServiceBaseCommand;
