import { BinaryDataService } from 'n8n-core';
import type { User } from '../databases/entities/User';
import type { WorkflowEntity } from '../databases/entities/WorkflowEntity';
import type { WorkflowSharingRole } from '../databases/entities/SharedWorkflow';
import { ExecutionRepository } from '../databases/repositories/execution.repository';
import { SharedWorkflowRepository } from '../databases/repositories/sharedWorkflow.repository';
import { WorkflowTagMappingRepository } from '../databases/repositories/workflowTagMapping.repository';
import { WorkflowRepository } from '../databases/repositories/workflow.repository';
import { ActiveWorkflowRunner } from '../ActiveWorkflowRunner';
import { ExternalHooks } from '../ExternalHooks';
import { type ListQuery } from '../requests';
import { TagService } from '../services/tag.service';
import { OwnershipService } from '../services/ownership.service';
import { WorkflowHistoryService } from './workflowHistory/workflowHistory.service.ee';
import { Logger } from '../Logger';
import { OrchestrationService } from '../services/orchestration.service';
export declare class WorkflowService {
    private readonly logger;
    private readonly executionRepository;
    private readonly sharedWorkflowRepository;
    private readonly workflowRepository;
    private readonly workflowTagMappingRepository;
    private readonly binaryDataService;
    private readonly ownershipService;
    private readonly tagService;
    private readonly workflowHistoryService;
    private readonly orchestrationService;
    private readonly externalHooks;
    private readonly activeWorkflowRunner;
    constructor(logger: Logger, executionRepository: ExecutionRepository, sharedWorkflowRepository: SharedWorkflowRepository, workflowRepository: WorkflowRepository, workflowTagMappingRepository: WorkflowTagMappingRepository, binaryDataService: BinaryDataService, ownershipService: OwnershipService, tagService: TagService, workflowHistoryService: WorkflowHistoryService, orchestrationService: OrchestrationService, externalHooks: ExternalHooks, activeWorkflowRunner: ActiveWorkflowRunner);
    getMany(sharedWorkflowIds: string[], options?: ListQuery.Options): Promise<{
        workflows: (Pick<WorkflowEntity, "id"> & Partial<Pick<WorkflowEntity, "name" | "tags" | "active" | "createdAt" | "updatedAt" | "versionId">>)[];
        count: number;
    }>;
    update(user: User, workflow: WorkflowEntity, workflowId: string, tagIds?: string[], forceSave?: boolean, roles?: WorkflowSharingRole[]): Promise<WorkflowEntity>;
    delete(user: User, workflowId: string): Promise<WorkflowEntity | undefined>;
}
