import type { CredentialsEntity } from '../databases/entities/CredentialsEntity';
import type { User } from '../databases/entities/User';
import type { WorkflowEntity } from '../databases/entities/WorkflowEntity';
import { CredentialsRepository } from '../databases/repositories/credentials.repository';
import { WorkflowRepository } from '../databases/repositories/workflow.repository';
import { SharedWorkflowRepository } from '../databases/repositories/sharedWorkflow.repository';
import { CredentialsService } from '../credentials/credentials.service';
import { Logger } from '../Logger';
import type { WorkflowWithSharingsAndCredentials } from './workflows.types';
export declare class EnterpriseWorkflowService {
    private readonly logger;
    private readonly sharedWorkflowRepository;
    private readonly workflowRepository;
    private readonly credentialsRepository;
    private readonly credentialsService;
    constructor(logger: Logger, sharedWorkflowRepository: SharedWorkflowRepository, workflowRepository: WorkflowRepository, credentialsRepository: CredentialsRepository, credentialsService: CredentialsService);
    isOwned(user: User, workflowId: string): Promise<{
        ownsWorkflow: boolean;
        workflow?: WorkflowEntity;
    }>;
    addOwnerAndSharings(workflow: WorkflowWithSharingsAndCredentials): void;
    addCredentialsToWorkflow(workflow: WorkflowWithSharingsAndCredentials, currentUser: User): Promise<void>;
    validateCredentialPermissionsToUser(workflow: WorkflowEntity, allowedCredentials: CredentialsEntity[]): void;
    preventTampering(workflow: WorkflowEntity, workflowId: string, user: User): Promise<WorkflowEntity>;
    validateWorkflowCredentialUsage(newWorkflowVersion: WorkflowEntity, previousWorkflowVersion: WorkflowEntity, credentialsUserHasAccessTo: CredentialsEntity[]): WorkflowEntity;
    getNodesWithInaccessibleCreds(workflow: WorkflowEntity, userCredIds: string[]): import("n8n-workflow").INode[];
}
