import express from 'express';
import type { IWorkflowResponse } from '../Interfaces';
import { WorkflowEntity } from '../databases/entities/WorkflowEntity';
import { SharedWorkflowRepository } from '../databases/repositories/sharedWorkflow.repository';
import { TagRepository } from '../databases/repositories/tag.repository';
import { WorkflowRepository } from '../databases/repositories/workflow.repository';
import { UserRepository } from '../databases/repositories/user.repository';
import { ExternalHooks } from '../ExternalHooks';
import { ListQuery } from '../requests';
import { WorkflowService } from './workflow.service';
import { License } from '../License';
import { InternalHooks } from '../InternalHooks';
import { TagService } from '../services/tag.service';
import { WorkflowHistoryService } from './workflowHistory/workflowHistory.service.ee';
import { Logger } from '../Logger';
import { NamingService } from '../services/naming.service';
import { UserOnboardingService } from '../services/userOnboarding.service';
import { CredentialsService } from '../credentials/credentials.service';
import { WorkflowRequest } from './workflow.request';
import { EnterpriseWorkflowService } from './workflow.service.ee';
import { WorkflowExecutionService } from './workflowExecution.service';
import { WorkflowSharingService } from './workflowSharing.service';
import { UserManagementMailer } from '../UserManagement/email';
export declare class WorkflowsController {
    private readonly logger;
    private readonly internalHooks;
    private readonly externalHooks;
    private readonly tagRepository;
    private readonly enterpriseWorkflowService;
    private readonly workflowHistoryService;
    private readonly tagService;
    private readonly namingService;
    private readonly userOnboardingService;
    private readonly workflowRepository;
    private readonly workflowService;
    private readonly workflowExecutionService;
    private readonly workflowSharingService;
    private readonly sharedWorkflowRepository;
    private readonly userRepository;
    private readonly license;
    private readonly mailer;
    private readonly credentialsService;
    constructor(logger: Logger, internalHooks: InternalHooks, externalHooks: ExternalHooks, tagRepository: TagRepository, enterpriseWorkflowService: EnterpriseWorkflowService, workflowHistoryService: WorkflowHistoryService, tagService: TagService, namingService: NamingService, userOnboardingService: UserOnboardingService, workflowRepository: WorkflowRepository, workflowService: WorkflowService, workflowExecutionService: WorkflowExecutionService, workflowSharingService: WorkflowSharingService, sharedWorkflowRepository: SharedWorkflowRepository, userRepository: UserRepository, license: License, mailer: UserManagementMailer, credentialsService: CredentialsService);
    create(req: WorkflowRequest.Create): Promise<WorkflowEntity>;
    getAll(req: ListQuery.Request, res: express.Response): Promise<void>;
    getNewName(req: WorkflowRequest.NewName): Promise<{
        name: string;
        onboardingFlowEnabled: boolean;
    }>;
    getFromUrl(req: WorkflowRequest.FromUrl): Promise<IWorkflowResponse>;
    getWorkflow(req: WorkflowRequest.Get): Promise<WorkflowEntity>;
    update(req: WorkflowRequest.Update): Promise<WorkflowEntity>;
    delete(req: WorkflowRequest.Delete): Promise<boolean>;
    runManually(req: WorkflowRequest.ManualRun): Promise<{
        waitingForWebhook: boolean;
        executionId?: undefined;
    } | {
        executionId: string;
        waitingForWebhook?: undefined;
    }>;
    share(req: WorkflowRequest.Share): Promise<void>;
}
